"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
Object.defineProperty(exports, "Router", {
  enumerable: true,
  get: function () {
    return _Router.default;
  }
});
exports.helpers = exports.default = void 0;

var _easevalidation = require("easevalidation");

var _oors = require("oors");

var _path = _interopRequireDefault(require("path"));

var _express = require("express");

var _ErrorWrapper = _interopRequireDefault(require("oors/build/errors/ErrorWrapper"));

var _validators = require("oors-express/build/validators");

var _Router = _interopRequireDefault(require("./libs/Router"));

var helpers = _interopRequireWildcard(require("./libs/helpers"));

exports.helpers = helpers;

var _generateRESTRouter = _interopRequireDefault(require("./libs/generateRESTRouter"));

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = Object.defineProperty && Object.getOwnPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : {}; if (desc.get || desc.set) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

class Router extends _oors.Module {
  constructor(..._args) {
    super(..._args);
    this.name = 'oors.router';

    this.addRouter = (...args) => {
      if (args.length === 2) {
        args.unshift('/');
      }

      const [pathPrefix, id, router] = args;
      this.deps['oors.express'].middlewares.insert(this.getConfig('middlewarePivot'), {
        path: pathPrefix,
        id,
        factory: () => router
      });
    };

    this.loadModuleRouter = module => {
      let moduleConfig;

      try {
        moduleConfig = (0, _easevalidation.validate)(_easevalidation.validators.isSchema({
          autoload: [_easevalidation.validators.isDefault(true), _easevalidation.validators.isBoolean()],
          prefixPath: [_easevalidation.validators.isDefault('/'), _easevalidation.validators.isString()]
        }))(module.getConfig('router', {}));
      } catch (error) {
        throw new _ErrorWrapper.default(error, `Invalid module "${module.name}" router configuration!`);
      }

      const {
        autoload,
        prefixPath
      } = moduleConfig;

      if (!autoload) {
        return;
      }

      let {
        router
      } = module;

      if (!router) {
        const routerPath = _path.default.resolve(_path.default.dirname(module.filePath), 'router');

        try {
          // eslint-disable-next-line import/no-dynamic-require, global-require
          router = require(routerPath).default;
        } catch (err) {} // eslint-disable-line no-empty

      }

      if (router) {
        this.addRouter(prefixPath, `${module.name}Router`, router);
      }
    };
  }

  async setup({
    autoload
  }) {
    await this.loadDependencies(['oors.express']);
    const {
      addRouter
    } = this;
    const router = (0, _express.Router)();

    if (autoload) {
      await this.runHook('load', this.loadModuleRouter, {
        addRouter,
        router
      });
    }

    addRouter('mainRouter', router);
    this.export({
      generateRESTRouter: _generateRESTRouter.default,
      addRouter,
      router
    });
  }

}

exports.default = Router;
Router.validateConfig = (0, _easevalidation.validate)(_easevalidation.validators.isSchema({
  middlewarePivot: (0, _validators.isMiddlewarePivot)(),
  autoload: [_easevalidation.validators.isDefault(true), _easevalidation.validators.isBoolean()]
}));