"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _express = require("express");

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(source, true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(source).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

class Router {
  static wrapHandler(handler) {
    return (req, res, next) => Promise.resolve(handler(req, res, next)).then(response => {
      if (typeof response !== 'undefined') {
        res.json(response);
      }
    }).catch(next);
  }

  constructor() {
    this.params = {};
    this.routes = [];
    this.routers = {};
    this.loadRoutes();
    this.middlewares = [];
    this.router = (0, _express.Router)();
  }

  loadRoutes() {
    for (const propr in this) {
      if (typeof this[propr] === 'function' && this[propr].isRoute) {
        const {
          definition
        } = this[propr];
        this.addRoute(_objectSpread({
          id: propr,
          path: `/${propr}`
        }, definition, {
          handler: this[propr].bind(this)
        }));
      }
    }
  }

  addRoute({
    id,
    method = 'GET',
    path = '/',
    handler,
    middlewares = []
  }) {
    this.routes.push({
      id,
      method,
      path,
      handler: this.constructor.wrapHandler(handler),
      middlewares
    });
    return this;
  }

  addParam(paramName, handler) {
    this.params[paramName] = this.constructor.wrapHandler(handler);
  }

  addRouter(path, router) {
    this.routers[path] = router;
  }

  getRoutes() {
    return this.routes;
  }

  use(middlewares) {
    this.middlewares.push(...(Array.isArray(middlewares) ? middlewares : [middlewares]));
  }

  toExpress() {
    if (this.middlewares.length) {
      this.router.use(this.middlewares);
    }

    Object.keys(this.params).forEach(paramName => {
      this.router.param(paramName, this.params[paramName]);
    });
    this.getRoutes().reduce((expressRouter, {
      method,
      path,
      middlewares,
      handler
    }) => {
      expressRouter[method](path, ...middlewares, handler);
      return expressRouter;
    }, this.router);
    Object.keys(this.routers).forEach(path => {
      const subRouter = this.routers[path].toExpress();
      this.router.use(path, subRouter);
    });
    return this.router;
  }

  mount(app, path = '/') {
    return app.use(path, this.toExpress());
  }

}

Router.route = definition => (target, propr, descriptor) => {
  Object.assign(descriptor, {
    enumerable: true
  });
  Object.assign(descriptor.value, {
    isRoute: true,
    definition
  });
  return descriptor;
};

['get', 'post', 'put', 'patch', 'delete', 'head'].forEach(method => {
  Router.route[method] = definition => Router.route(_objectSpread({}, definition, {
    method
  }));
});
var _default = Router;
exports.default = _default;