"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _express = require("express");

var _boom = _interopRequireDefault(require("boom"));

var _mongodb = require("mongodb");

var _pick = _interopRequireDefault(require("lodash/pick"));

var _easevalidation = require("easevalidation");

var _helpers = require("./helpers");

var _validate = _interopRequireDefault(require("../middlewares/validate"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(source, true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(source).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var _default = ({
  router = (0, _express.Router)(),
  repository
}) => {
  router.param('id', async (req, res, next, rawId) => {
    let id;

    try {
      id = (0, _mongodb.ObjectID)(rawId);
    } catch (error) {
      return next(error);
    }

    try {
      const entity = await repository.findById(id);

      if (!entity) {
        return next(_boom.default.notFound(`Unable to find entity with id ${rawId}`));
      }

      req.entity = entity;
    } catch (err) {
      return next(err);
    }

    return next();
  });
  router.route('/').get((0, _validate.default)({
    query: [_easevalidation.validators.isDefault({}), _easevalidation.validators.isSchema({
      query: [_easevalidation.validators.isDefault({}), _easevalidation.validators.isObject()],
      offset: _easevalidation.validators.isAny(_easevalidation.validators.isUndefined(), _easevalidation.validators.isNumber()),
      limit: _easevalidation.validators.isAny(_easevalidation.validators.isUndefined(), _easevalidation.validators.isNumber()),
      orderBy: [_easevalidation.validators.isDefault({}), _easevalidation.validators.isObject()],
      fields: [_easevalidation.validators.isDefault({}), _easevalidation.validators.isObject()]
    })]
  }), (0, _helpers.wrapHandler)(async req => {
    const params = (0, _pick.default)(req.query, ['fields', 'orderBy']);
    params.query = (0, _helpers.toBSON)(req.query.query);

    if (req.query.offset !== undefined) {
      params.skip = parseInt(req.query.offset, 10);
    }

    if (req.query.limit !== undefined) {
      params.limit = parseInt(req.query.limit, 10);
    }

    return repository.findMany(params);
  })).post((0, _helpers.wrapHandler)(req => repository.createOne((0, _helpers.toBSON)(req.body))));
  router.get('/count', (0, _validate.default)({
    query: [_easevalidation.validators.isDefault({}), _easevalidation.validators.isSchema({
      query: [_easevalidation.validators.isDefault({}), _easevalidation.validators.isObject()]
    })]
  }), (0, _helpers.wrapHandler)(req => repository.count({
    query: (0, _helpers.toBSON)(req.query.query)
  })));
  router.delete('/deleteOne', (0, _validate.default)({
    query: _easevalidation.validators.isSchema({
      query: [_easevalidation.validators.isRequired(), _easevalidation.validators.isObject()]
    })
  }), (0, _helpers.wrapHandler)(req => repository.deleteOne({
    query: (0, _helpers.toBSON)(req.query.query)
  })));
  router.route('/:id').get((req, res) => res.json(req.entity)).put((0, _helpers.wrapHandler)(req => repository.replaceOne({
    query: {
      _id: req.entity._id
    },
    replacement: _objectSpread({}, req.entity, {}, (0, _helpers.toBSON)(req.body))
  }))).patch((0, _helpers.wrapHandler)(async req => {
    await repository.validate(_objectSpread({}, req.entity, {}, req.body));
    return repository.updateOne({
      query: {
        _id: req.entity._id
      },
      update: {
        $set: req.body
      }
    });
  })).delete((0, _helpers.wrapHandler)(req => repository.deleteOne({
    query: {
      _id: req.entity._id
    }
  })));
  router.get('/findOne', (0, _validate.default)({
    query: [_easevalidation.validators.isDefault({}), _easevalidation.validators.isSchema({
      query: [_easevalidation.validators.isDefault({}), _easevalidation.validators.isObject()]
    })]
  }), (0, _helpers.wrapHandler)(async req => {
    const query = (0, _helpers.toBSON)(req.query.query);
    const entity = await repository.findOne({
      query
    });

    if (!entity) {
      throw _boom.default.notFound('Unable to find entity.');
    }

    return entity;
  }));
  return router;
};

exports.default = _default;